# -*- coding: utf-8 -*-

###########################
# Thumbnails Cleaner      #
# by Max (m4x1m) Headroom #
###########################

import os, unicodedata, urllib, datetime, time, shutil, gui, xbmc, xbmcaddon, xbmcgui
import simplejson as json
from dbutils import RawXbmcDb

addonSettings = xbmcaddon.Addon( "script.thumbnailscleaner" )
addonName     = addonSettings.getAddonInfo( "name" )
addonLanguage = addonSettings.getLocalizedString
addonIcon     = xbmc.translatePath( os.path.join( addonSettings.getAddonInfo( "path" ), "icon.png" ) )

rootPath         = os.path.dirname( os.path.dirname( addonSettings.getAddonInfo( "path" ) ) )
thumbnailsFolder = xbmc.translatePath( os.path.join( rootPath, "userdata", "Thumbnails" ) )
databaseFolder   = xbmc.translatePath( os.path.join( rootPath, "userdata", "Database" ) )

def selectDestinationFolder():
     destinationFolder = xbmc.translatePath( addonSettings.getSetting( "destinationFolder" ) )
     if not destinationFolder or not os.path.exists( destinationFolder ):
	  addonSettings.openSettings()
	  destinationFolder = selectDestinationFolder()
     return destinationFolder

def log( msg ):
     xbmc.log( "[%s] - %s" % ( addonName, msg ) )

def _unicode( text, encoding='utf-8' ):
    try: text = unicode( text, encoding )
    except: pass
    return text

def normalize( text ):
     try: text = unicodedata.normalize( 'NFKD', _unicode( text ) ).encode( 'utf-8' )
     except:pass
     return text

def getHash( string ):
     string = string.lower()
     bytes = bytearray( string )
     crc = 0xffffffff;
     for b in bytes:
	  crc = crc ^ ( b << 24 )
	  for i in range(8):
	       if ( crc & 0x80000000 ): crc = ( crc << 1 ) ^ 0x04C11DB7
	       else: crc = crc << 1;
	  crc = crc & 0xFFFFFFFF
     return '%08x' % crc

def humanReadableSizeOf( size ):
     for x in ['bytes','KB','MB','GB']:
	  if size < 1024.0 and size > -1024.0:
	       return "%3.1f%s" % ( size, x )
	  size /= 1024.0
     return "%3.1f%s" % ( size, 'TB' )

def removeDuplicate( lists ):
     seen = set()
     seenAdd = seen.add
     return [ x for x in lists if x not in seen and not seenAdd( x ) ]

class Cleaner:
     def __init__( self, parent ):
	  self.parent           = parent
	  self.listFiles        = []
	  self.totalFileSize    = 0
	  self.cancelOperation  = False
	  self.destinationFolder = selectDestinationFolder()

     def queryTable( self, getQuery='' ):
	  lists = []
	  if not getQuery:
	       keyboard = xbmc.Keyboard()
	       keyboard.doModal()
	       getQuery = keyboard.getText()
	  if getQuery:
	       match = RawXbmcDb.query( "SELECT url FROM texture WHERE url LIKE '%" + getQuery + "%'" )
	       for base in match: lists.append( base[0] )
	       if len( lists ) > 0:
		    dialog = xbmcgui.Dialog()
		    result = dialog.select( normalize( addonLanguage(30400) ), lists )
		    if result != -1:
			 RawXbmcDb.query( "DELETE FROM texture WHERE url='" + lists[result] + "'" )
			 self.setLabelInfo()
			 self.queryTable( getQuery )
	       else: self.queryTable()

     def emptyTable( self ):
	  self.Erase = xbmcgui.Dialog().yesno( "%s" % addonName, "%s" % normalize( addonLanguage(30401) ), "%s" % normalize( addonLanguage(30402) ), " ", "%s" % "No", "%s" % "Yes" )
	  if self.Erase != 0:
	       shutil.copy2(databaseFolder + "/Textures" + addonSettings.getSetting( "dbTextures" ) + ".db", databaseFolder + "/Textures" + addonSettings.getSetting( "dbTextures" ) + ".db.bak")
	       RawXbmcDb.query( "DELETE FROM texture" )
	       log( normalize( addonLanguage(30403) ) )
	       log( normalize( addonLanguage(30404) ) )
	       xbmcgui.Dialog().ok( "%s - %s" % ( addonName, normalize( "Info" ) ), "%s" % normalize( addonLanguage(30403) ), normalize( addonLanguage(30404) ) )
	       self.setLabelInfo()

     def setLabelInfo( self ):
	  # Get all files in Thumbnails folder and read Textures database
	  self.exploreThumbnailsFolder( thumbnailsFolder )
	  match = RawXbmcDb.query( "SELECT cachedurl, lasthashcheck FROM texture" )
	  # set Labels
	  self.parent.labelBeforeFiles.setLabel( str( len( self.listFiles ) ) )
	  self.parent.labelBeforeSizes.setLabel( humanReadableSizeOf( self.totalFileSize ) )
	  self.parent.labelBeforeTextures.setLabel( str( len( match ) ) )

     def doClean( self, doSimulate=True ):
	  # Reset After Labels
	  self.parent.labelAfterFiles.setLabel( "" )
	  self.parent.labelAfterSizes.setLabel( "" )
	  self.parent.labelAfterTextures.setLabel( "" )
	  # Disable Buttons
	  self.parent.buttonClean.setEnabled(False)
	  self.parent.buttonSimulate.setEnabled(False)
	  self.parent.buttonQuery.setEnabled(False)
	  self.parent.buttonEmpty.setEnabled(False)
	  self.parent.buttonSettings.setEnabled(False)
	  self.startedAt = datetime.datetime.now()
	  if doSimulate == True: log( normalize( addonLanguage(30200) ) + self.startedAt.strftime( "%Y-%m-%d %H:%M:%d" ) )
	  else: log( normalize( addonLanguage(30201) ) + self.startedAt.strftime( "%Y-%m-%d %H:%M:%d" ) )
	  # get All Files in Thumbnails Folder
	  self.totalListFiles = 0
	  self.exploreThumbnailsFolder( thumbnailsFolder )
	  self.totalListFiles = len( self.listFiles )

	  # Movies
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30204) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataMovie = []; countList = 1
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetMovies", "params": {"properties": ["art"], "sort": { "order": "ascending", "method": "title", "ignorearticle": true } }, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'movies' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30205) % totalResults ) )
		    for item in jSon['result']['movies']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30204) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 for key in item.get( 'art' ):
			      valueText = urllib.unquote_plus( normalize( item.get('art')[key] ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataMovie.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataMovie = removeDuplicate( self.dataMovie )
	       if self.cancelOperation != True and len( self.dataMovie ) > 0:
		    self.excludeHash( self.generateHash( self.dataMovie ), normalize( addonLanguage(30206) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Sets
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30207) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataSets = []; countList = 1
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetMovieSets", "params": {"properties": ["art"], "sort": { "order": "ascending", "method": "title", "ignorearticle": true } }, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'sets' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30208) % totalResults ) )
		    for item in jSon['result']['sets']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30207) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 for key in item.get( 'art' ):
			      valueText = urllib.unquote_plus( normalize( item.get('art')[key] ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataSets.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataSets = removeDuplicate( self.dataSets )
	       if self.cancelOperation != True and len( self.dataSets ) > 0:
		    self.excludeHash( self.generateHash( self.dataSets ), normalize( addonLanguage(30209) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # TVShows
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30210) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataTVShows = []; self.tvShows = []; countList = 1
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetTVShows", "params": {"properties": ["art"], "sort": { "order": "ascending", "method": "title", "ignorearticle": true } }, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'tvshows' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30211) % totalResults ) )
		    for item in jSon['result']['tvshows']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30210) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 self.tvShows.append( ( item.get('tvshowid'), item.get('label') ) )
			 for key in item.get( 'art' ):
			      valueText = urllib.unquote_plus( normalize( item.get('art')[key] ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataTVShows.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataTVShows = removeDuplicate( self.dataTVShows )
	       if self.cancelOperation != True and len( self.dataTVShows ) > 0:
		    self.excludeHash( self.generateHash( self.dataTVShows ), normalize( addonLanguage(30212) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Seasons
	  if self.cancelOperation != True and len( self.tvShows ) > 0:
	       log( normalize( "*** " + addonLanguage(30213) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataSeasons = []; countList = 1
	       Progress = xbmcgui.DialogProgress()
	       Progress.create( addonName )
	       for tvShow in self.tvShows:
		    Progress.update( ( countList*100 )/len( self.tvShows ), normalize( addonLanguage(30213) ), normalize( tvShow[1] ), " " )
		    if Progress.iscanceled():
			 Progress.close()
			 self.cancelOperation = True
			 break
		    jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetSeasons", "params": {"properties": ["art"], "tvshowid": %s}, "id": 1}' % tvShow[0] )
		    jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
		    jSon = json.loads( jsonQuery )
		    if jSon['result'].has_key( 'seasons' ):
			 totalResults = jSon['result']['limits'].get('total')
			 log( normalize( addonLanguage(30214) % ( totalResults, tvShow[1] ) ) )
			 for item in jSon['result']['seasons']:
			      for key in item.get( 'art' ):
				   if key.find("tvshow.") != 0:
					valueText = urllib.unquote_plus( normalize( item.get('art')[key] ) )
					valueText = valueText.replace( "image://", "" )[:-1]
					if item.get('label') == "Season 1": # Workaround for season all posters
					     seasonAll = valueText.replace( "season01", "season-all" )
					     self.dataSeasons.append( seasonAll )
					self.dataSeasons.append( valueText )
		    time.sleep(0.0005)
		    countList = countList + 1
	       self.dataSeasons = removeDuplicate( self.dataSeasons )
	       if self.cancelOperation != True and len( self.dataSeasons ) > 0:
		    self.excludeHash( self.generateHash( self.dataSeasons ), normalize( addonLanguage(30215) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Episodes
	  if self.cancelOperation != True and len( self.tvShows ) > 0:
	       log( normalize( "*** " + addonLanguage(30216) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataEpisodes = []; countList = 1
	       Progress = xbmcgui.DialogProgress()
	       Progress.create( addonName )
	       for tvShow in self.tvShows:
		    Progress.update( ( countList*100 )/len( self.tvShows ), normalize( addonLanguage(30216) ), normalize( tvShow[1] ), " " )
		    if Progress.iscanceled():
			 Progress.close()
			 self.cancelOperation = True
			 break
		    jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetEpisodes", "params": {"properties": ["art"], "tvshowid": %s}, "id": 1}' % tvShow[0] )
		    jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
		    jSon = json.loads( jsonQuery )
		    if jSon['result'].has_key( 'episodes' ):
			 totalResults = jSon['result']['limits'].get('total')
			 log( normalize( addonLanguage(30217) % ( totalResults, tvShow[1] ) ) )
			 for item in jSon['result']['episodes']:
			      for key in item.get( 'art' ):
				   if key.find("tvshow.") != 0:
					valueText = urllib.unquote_plus( normalize( item.get('art')[key] ) )
					valueText = valueText.replace( "image://", "" )[:-1]
					self.dataEpisodes.append( valueText )
		    time.sleep(0.0005)
		    countList = countList + 1
	       self.dataEpisodes = removeDuplicate( self.dataEpisodes )
	       if self.cancelOperation != True and len( self.dataEpisodes ) > 0:
		    self.excludeHash( self.generateHash( self.dataEpisodes ), normalize( addonLanguage(30218) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # MusicVideos
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30219) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataMusicVideos = []; countList = 1
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetMusicVideos", "params": {"properties": ["art"], "sort": { "order": "ascending", "method": "title", "ignorearticle": true } }, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'musicvideos' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30220) % totalResults ) )
		    for item in jSon['result']['musicvideos']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30219) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 for key in item.get( 'art' ):
			      valueText = urllib.unquote_plus( normalize( item.get('art')[key] ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataMusicVideos.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataMusicVideos = removeDuplicate( self.dataMusicVideos )
	       if self.cancelOperation != True and len( self.dataMusicVideos ) > 0:
		    self.excludeHash( self.generateHash( self.dataMusicVideos ), normalize( addonLanguage(30221) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Video Genres
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30222) + " ***" ) )
	       self.dataVideoGenres = []
	       # Movies
	       countList = 1
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetGenres", "params": {"sort": { "order": "ascending", "method": "label", "ignorearticle": true }, "properties": ["thumbnail"], "type": "movie"}, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'genres' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30224) % totalResults ) )
		    for item in jSon['result']['genres']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30223) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 if item.get( 'thumbnail' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'thumbnail' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataVideoGenres.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       # TV Shows
	       countList = 1
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetGenres", "params": {"sort": { "order": "ascending", "method": "label", "ignorearticle": true }, "properties": ["thumbnail"], "type": "tvshow"}, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'genres' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30226) % totalResults ) )
		    for item in jSon['result']['genres']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30225) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 if item.get( 'thumbnail' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'thumbnail' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataVideoGenres.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       # Music Videos
	       countList = 1
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetGenres", "params": {"sort": { "order": "ascending", "method": "label", "ignorearticle": true }, "properties": ["thumbnail"], "type": "musicvideo"}, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'genres' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30228) % totalResults ) )
		    for item in jSon['result']['genres']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30227) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 if item.get( 'thumbnail' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'thumbnail' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataVideoGenres.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataVideoGenres = removeDuplicate( self.dataVideoGenres )
	       if self.cancelOperation != True and len( self.dataVideoGenres ) > 0:
		    self.excludeHash( self.generateHash( self.dataVideoGenres ), normalize( addonLanguage(30229) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Music Artists
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30230) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataMusicArtists = []; countList = 1
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "AudioLibrary.GetArtists", "params": {"properties": ["thumbnail","fanart"], "sort": { "order": "ascending", "method": "title", "ignorearticle": true } }, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'artists' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30231) % totalResults ) )
		    for item in jSon['result']['artists']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30230) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 if item.get( 'thumbnail' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'thumbnail' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataMusicArtists.append( valueText )
			 if item.get( 'fanart' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'fanart' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataMusicArtists.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataMusicArtists = removeDuplicate( self.dataMusicArtists )
	       if self.cancelOperation != True and len( self.dataMusicArtists ) > 0:
		    self.excludeHash( self.generateHash( self.dataMusicArtists ), normalize( addonLanguage(30232) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Music Albums
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30233) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataMusicAlbums = []; countList = 1
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "AudioLibrary.GetAlbums", "params": {"properties": ["thumbnail","fanart"], "sort": { "order": "ascending", "method": "title", "ignorearticle": true } }, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'albums' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30234) % totalResults ) )
		    for item in jSon['result']['albums']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30233) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 if item.get( 'thumbnail' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'thumbnail' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataMusicAlbums.append( valueText )
			 if item.get( 'fanart' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'fanart' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataMusicAlbums.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataMusicAlbums = removeDuplicate( self.dataMusicAlbums )
	       if self.cancelOperation != True and len( self.dataMusicAlbums ) > 0:
		    self.excludeHash( self.generateHash( self.dataMusicAlbums ), normalize( addonLanguage(30235) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Music Songs
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30236) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataMusicSongs = []; countList = 1
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "AudioLibrary.GetSongs", "params": {"properties": ["thumbnail","fanart"], "sort": { "order": "ascending", "method": "title", "ignorearticle": true } }, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'songs' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30237) % totalResults ) )
		    for item in jSon['result']['songs']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30236) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 if item.get( 'thumbnail' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'thumbnail' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataMusicSongs.append( valueText )
			 if item.get( 'fanart' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'fanart' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataMusicSongs.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataMusicSongs = removeDuplicate( self.dataMusicSongs )
	       if self.cancelOperation != True and len( self.dataMusicSongs ) > 0:
		    self.excludeHash( self.generateHash( self.dataMusicSongs ), normalize( addonLanguage(30238) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Music Genres
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30239) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataMusicGenres = []; countList = 1
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "AudioLibrary.GetGenres", "params": {"sort": { "order": "ascending", "method": "title", "ignorearticle": true }, "properties": ["thumbnail"]}, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'genres' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30240) % totalResults ) )
		    for item in jSon['result']['genres']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30239) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 if item.get( 'thumbnail' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'thumbnail' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataMusicGenres.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataMusicGenres = removeDuplicate( self.dataMusicGenres )
	       if self.cancelOperation != True and len( self.dataMusicGenres ) > 0:
		    self.excludeHash( self.generateHash( self.dataMusicGenres ), normalize( addonLanguage(30241) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Actors
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30245) + " ***" ) )
	       self.dataCast = []
	       # Movies
	       countList = 1
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetMovies", "params": {"properties": ["cast"], "sort": { "order": "ascending", "method": "title", "ignorearticle": true } }, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'movies' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30205) % totalResults ) )
		    for item in jSon['result']['movies']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30245) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 for cast in item.get( 'cast' ):
			      for key in cast:
				   if key == "thumbnail":
					valueText = urllib.unquote_plus( normalize( cast[key] ) )
					valueText = valueText.replace( "image://", "" )[:-1]
					self.dataCast.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       # TV Shows
	       self.tvShows = []; countList = 1
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetTVShows", "params": {"properties": ["cast"], "sort": { "order": "ascending", "method": "title", "ignorearticle": true } }, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'tvshows' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30211) % totalResults ) )
		    for item in jSon['result']['tvshows']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30246) ), normalize( item.get('label') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 self.tvShows.append( ( item.get('tvshowid'), item.get('label') ) )
			 for cast in item.get( 'cast' ):
			      for key in cast:
				   if key == "thumbnail":
					valueText = urllib.unquote_plus( normalize( cast[key] ) )
					valueText = valueText.replace( "image://", "" )[:-1]
					self.dataCast.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       # Episodes
	       countList = 1
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       Progress = xbmcgui.DialogProgress()
	       Progress.create( addonName )
	       for tvShow in self.tvShows:
		    Progress.update( ( countList*100 )/len( self.tvShows ), normalize( addonLanguage(30247) ), normalize( tvShow[1] ), " " )
		    if Progress.iscanceled():
			 Progress.close()
			 self.cancelOperation = True
			 break
		    jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "VideoLibrary.GetEpisodes", "params": {"properties": ["cast"], "tvshowid": %s}, "id": 1}' % tvShow[0] )
		    jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
		    jSon = json.loads( jsonQuery )
		    if jSon['result'].has_key( 'episodes' ):
			 totalResults = jSon['result']['limits'].get('total')
			 log( normalize( addonLanguage(30217) % ( totalResults, tvShow[1] ) ) )
			 for item in jSon['result']['episodes']:
			      for cast in item.get( 'cast' ):
				   for key in cast:
					if key == "thumbnail":
					     valueText = urllib.unquote_plus( normalize( cast[key] ) )
					     valueText = valueText.replace( "image://", "" )[:-1]
					     self.dataCast.append( valueText )
		    time.sleep(0.0005)
		    countList = countList + 1
	       self.dataCast = removeDuplicate( self.dataCast )
	       if self.cancelOperation != True and len( self.dataCast ) > 0:
		    self.excludeHash( self.generateHash( self.dataCast ), normalize( addonLanguage(30248) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Addons
	  if self.cancelOperation != True:
	       log( normalize( "*** " + addonLanguage(30242) + " ***" ) )
	       log( normalize( addonLanguage(30203) ) )
	       xbmc.executebuiltin( "Notification(%s,%s,5000,%s)" % ( addonName, normalize( addonLanguage(30203) ), addonIcon ) )
	       self.dataAddons = []; countList = 1
	       jsonQuery = xbmc.executeJSONRPC( '{"jsonrpc": "2.0", "method": "Addons.GetAddons", "params": {"properties": ["thumbnail","fanart"]}, "id": 1}' )
	       jsonQuery = unicode( jsonQuery, 'utf-8', errors='ignore' )
	       jSon = json.loads( jsonQuery )
	       if jSon['result'].has_key( 'addons' ):
		    Progress = xbmcgui.DialogProgress()
		    Progress.create( addonName )
		    totalResults = jSon['result']['limits'].get('total')
		    log( normalize( addonLanguage(30243) % totalResults ) )
		    for item in jSon['result']['addons']:
			 Progress.update( ( countList*100 )/totalResults, normalize( addonLanguage(30242) ), normalize( item.get('addonid') ), " " )
			 if Progress.iscanceled():
			      Progress.close()
			      self.cancelOperation = True
			      break
			 if item.get( 'thumbnail' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'thumbnail' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataAddons.append( valueText )
			 if item.get( 'fanart' ) != "":
			      valueText = urllib.unquote_plus( normalize( item.get( 'fanart' ) ) )
			      valueText = valueText.replace( "image://", "" )[:-1]
			      self.dataAddons.append( valueText )
			 time.sleep(0.0005)
			 countList = countList + 1
	       self.dataAddons = removeDuplicate( self.dataAddons )
	       if self.cancelOperation != True and len( self.dataAddons ) > 0:
		    self.excludeHash( self.generateHash( self.dataAddons ), normalize( addonLanguage(30244) ) )
	       else: log( normalize( addonLanguage(30301) ) % "0" )

	  # Process Textures Database
	  if self.cancelOperation != True:
	       self.allTextures = []
	       self.totalTextures = 0
	       self.deletedTextures = 0
	       # get All Textures
	       match = RawXbmcDb.query( "SELECT cachedurl, lasthashcheck FROM texture" )
	       self.totalTextures = len( match )
	       if doSimulate == True:
		    if addonSettings.getSetting( "removeEmpty" ) == "true":
			 match = RawXbmcDb.query( "SELECT cachedurl, lasthashcheck FROM texture WHERE lasthashcheck != ''" )
	       else:
		    if addonSettings.getSetting( "removeEmpty" ) == "true":
			 RawXbmcDb.query( "DELETE FROM texture WHERE lasthashcheck = ''" )
			 match = RawXbmcDb.query( "SELECT cachedurl, lasthashcheck FROM texture" )
	       self.deletedTextures = self.totalTextures - len( match )
	       self.totalTextures = len( match )
	       for base in match: self.allTextures.append( ( base[0], base[1] ) )
	       # Exclude Textures Files
	       hashTextures = []
	       for t in self.allTextures: hashTextures.append( t[0].split("/")[1].encode('utf-8') )
	       log( normalize( "*** " + addonLanguage(30249) + " ***" ) )
	       self.excludeHash( hashTextures, normalize( addonLanguage(30250) ) )
	       # To the end
	       if self.cancelOperation != True:
		    # Re-Calculate Total Files and Sizes
		    self.newTotalFileSize = 0
		    self.totalListFiles = self.totalListFiles-len( self.listFiles )
		    for s in self.listFiles:
			 self.newTotalFileSize = self.newTotalFileSize + os.stat( s[1] ).st_size
		    self.totalFileSize = self.totalFileSize - self.newTotalFileSize
		    # Print After Labels
		    self.parent.labelAfterFiles.setLabel( str ( self.totalListFiles ) )
		    self.parent.labelAfterSizes.setLabel( humanReadableSizeOf( self.totalFileSize ) )
		    self.parent.labelAfterTextures.setLabel( str( self.totalTextures ) )
		    # Finalize Clean
		    self.finalizeClean( doSimulate )
	       else:
		    log( normalize( addonLanguage(30501) ) )
		    xbmcgui.Dialog().ok( "%s - %s" % ( addonName, normalize( addonLanguage(30500) ) ), "%s" % normalize( addonLanguage(30501) ) )
	  else:
	       log( normalize( addonLanguage(30501) ) )
	       xbmcgui.Dialog().ok( "%s - %s" % ( addonName, normalize( addonLanguage(30500) ) ), "%s" % normalize( addonLanguage(30501) ) )
	  # Enable Buttons
	  self.parent.buttonClean.setEnabled(True)
	  self.parent.buttonSimulate.setEnabled(True)
	  self.parent.buttonQuery.setEnabled(True)
	  self.parent.buttonEmpty.setEnabled(True)
	  self.parent.buttonSettings.setEnabled(True)

     def exploreThumbnailsFolder( self, thumbnailsFolder ):
	  basedir = thumbnailsFolder
	  subdirlist = []
	  for item in os.listdir(thumbnailsFolder):
	       if os.path.isfile(os.path.join(basedir,item)):
		    last = os.path.split(os.path.dirname(basedir))[1]
		    itemWithDir = basedir + "/" + item
		    self.listFiles.append( ( item, itemWithDir ) )
		    self.totalFileSize = self.totalFileSize + os.stat( os.path.join( basedir,item ) ).st_size
	       else:
		    subdirlist.append(os.path.join(basedir, item))
	  for subdir in subdirlist:
	       self.exploreThumbnailsFolder(subdir)

     def generateHash( self, sectionToDo ):
	  hashing = []
	  for s in sectionToDo: hashing.append( getHash( s ) )
	  return hashing

     def excludeHash( self, sectionToDo, textToDo ):
	  newlistFiles = []; countList = 1
	  Progress = xbmcgui.DialogProgress()
	  Progress.create( addonName )
	  log( normalize( addonLanguage(30300) % len( sectionToDo ) ) )
	  for list in self.listFiles:
	       Progress.update( ( countList*100 )/len( self.listFiles ), normalize( addonLanguage(30301) ) % len( sectionToDo ), textToDo, list[0] )
	       if Progress.iscanceled():
		    Progress.close()
		    self.cancelOperation = True
		    break
	       find = False
	       for hashed in sectionToDo:
		    if list[0].split(".")[0] == hashed.split(".")[0]: find = True
	       if find == False: newlistFiles.append( ( list[0], list[1] ) )
	       countList = countList + 1
	  self.listFiles = newlistFiles

     def finalizeClean( self, doSimulate=True ):
	  # Move files in destination folder or copy if simulate is active
	  countList = 1
	  Progress = xbmcgui.DialogProgress()
	  Progress.create( addonName )
	  for c in self.listFiles:
	       if doSimulate == True:
		    Progress.update( ( countList*100 )/len( self.listFiles ), normalize( addonLanguage(30160) ) % len( self.listFiles ), c[1], " " )
		    shutil.copy2(c[1], self.destinationFolder)
	       else:
		    Progress.update( ( countList*100 )/len( self.listFiles ), normalize( addonLanguage(30161) ) % len( self.listFiles ), c[1], " " )
		    file = os.path.basename(c[1])
		    try: os.remove(os.path.join( self.destinationFolder, file ) )
		    except: pass
		    shutil.move(c[1], self.destinationFolder)
	  # Manage Textual Response
	  labelFile = normalize( addonLanguage(30162) )
	  labelField = normalize( addonLanguage(30163) )
	  typeOperation = normalize( addonLanguage(30164) )
	  typeOperationDB = normalize( addonLanguage(30165) )
	  typeRecovered = addonLanguage(30166)
	  if len( self.listFiles ) == 1: labelFile = normalize( addonLanguage(30167) )
	  if self.deletedTextures == 1: labelField = normalize( addonLanguage(30168) )
	  if doSimulate == True:
	       typeOperation = normalize( addonLanguage(30169) )
	       typeOperationDB = normalize( addonLanguage(30170) )
	       typeRecovered = addonLanguage(30171)
	  # Log in XBMC.LOG
	  log( "%s %s %s %s %s" % ( typeOperation, str( len( self.listFiles ) ), labelFile, normalize( typeRecovered ), humanReadableSizeOf( self.newTotalFileSize ) ) )
	  log( "%s %s %s %s" % ( self.deletedTextures, labelField, typeOperationDB, normalize( addonLanguage(30172) ) ) )
	  self.finishAt = datetime.datetime.now()
	  self.tookTime = self.finishAt - self.startedAt
	  log( normalize( addonLanguage(30502) % self.tookTime ) )
	  # Output Results
	  xbmcgui.Dialog().ok( "%s - %s" % ( addonName, normalize( addonLanguage(30173) ) ), "%s %s" % ( normalize( addonLanguage(30174) ), addonName ), "%s %s %s %s %s" % ( typeOperation, str( len( self.listFiles ) ), labelFile, typeRecovered, humanReadableSizeOf( self.newTotalFileSize ) ), "%s %s %s %s" % ( self.deletedTextures, labelField, typeOperationDB, normalize( addonLanguage(30172) ) ) )
	  # Enable Buttons
	  self.parent.buttonClean.setEnabled(True)
	  self.parent.buttonSimulate.setEnabled(True)
	  self.parent.buttonSettings.setEnabled(True)
	  self.parent.buttonExit.setEnabled(True)

if ( __name__ == "__main__" ):
     gui.GUI( "script_thumbnails_cleaner-main.xml", addonSettings.getAddonInfo( "path" ), "Default" )